<?php
/**
 * Detector library
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		3.0.0
 * @copyright	Copyright (c) 2009 Cracker Tracker Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// Initializes all detector arrays
function initCrackerTrackerArrays () {
	// Set error_reporting
	if (isCrackerTrackerDebug()) {
		// For debugging purposes, this is fine
		@error_reporting(E_ALL, E_STRICT);
	} else {
		// No output
		@error_reporting(0);
	}

	// Whitelist some absolute query strings (see below)
	$GLOBALS['ctracker_whitelist'] = array(
		'cmd=new',   // LinPHA
		'cmd=edit',  // LinPHA
		'cmd=lostpw' // LinPHA
	);

	// Attacks we should detect and blok
	$GLOBALS['ctracker_get_blacklist'] = array(
		'chr(', 'chr=', 'chr%20', '%20chr', 'wget%20', '%20wget', 'wget(',
		'cmd=', '%20cmd', 'cmd%20', 'rush=', '%20rush', 'rush%20',
		'union%20', '%20union', 'union(', 'union=', 'echr(', '%20echr', 'echr%20', 'echr=',
		'esystem(', 'esystem%20', 'cp%20', '%20cp', 'cp(', 'mdir%20', '%20mdir', 'mdir(',
		'mcd%20', 'mrd%20', 'rm%20', '%20mcd', '%20mrd', '%20rm',
		'mcd(', 'mrd(', 'rm(', 'mcd=', 'mrd=', 'mv%20', 'rmdir%20', 'mv(', 'rmdir(',
		'chmod(', 'chmod%20', '%20chmod', 'chmod(', 'chmod=', 'chown%20', 'chgrp%20', 'chown(', 'chgrp(',
		'locate%20', 'grep%20', 'locate(', 'grep(', 'diff%20', 'kill%20', 'kill(', 'killall',
		'passwd%20', '%20passwd', 'passwd(', 'telnet%20', 'vi(', 'vi%20', 'cgi-', '.eml',
		'insert%20into', 'select%20', 'nigga(', '%20nigga', 'nigga%20', 'fopen', 'fwrite', '%20like', 'like%20',
		'$_request', '$_get', '$request', '$get', '.system', 'HTTP_PHP', '&aim', '%20getenv', 'getenv%20',
		'new_password', '&icq','/etc/passwd','/etc/shadow', '/etc/groups', '/etc/gshadow',
		'HTTP_USER_AGENT', 'HTTP_HOST', 'wget%20', 'uname\x20-', 'uname%20-', 'bin/id', '/bin/', '/chgrp',
		'/chown', '/usr/bin', 'g\+\+', 'bin/python', 'bin/tclsh', 'bin/nasm', 'perl%20', 'traceroute%20',
		'ping%20', '.pl', 'bin/xterm', 'lsof%20', '.conf', 'motd%20', 'HTTP/1.', '.inc.php', '.lib.php',
		'config.php', 'file\://', 'window.open', '<SCRIPT>', 'javascript\://', 'img src', 'img%20src', '.jsp',
		'ftp.exe', 'xp_enumdsn', 'xp_availablemedia', 'xp_filelist', 'xp_cmdshell', 'nc.exe', '.htpasswd',
		'servlet', '/etc/passwd', 'wwwacl', '~root', '~ftp', '.js', '.jsp', '.history', 'bash_history',
		'.bash_history', '~nobody', 'server-info', 'server-status', 'reboot%20', 'halt%20',
		'powerdown%20', '/home/ftp', '/home/www', 'secure_site, ok', 'chunked', 'org.apache', '/servlet/con',
		'<script', '/robot.txt' ,'/perl' ,'mod_gzip_status', 'db_mysql.inc', '.inc', 'select%20from',
		'select from', 'drop%20', '.system', 'getenv', 'http_', '_php', 'php_', 'phpinfo()', '\<?php', '?\>', 'sql=',
		'div style=', 'overflow: auto', 'height: 1px', 'cc%20', 'admin_action=', 'path=', 'action=http',
		'page=http', 'module=http', 'op=http', 'id=http', 'id%3Dhttp', 'action%3Dhttp', 'page%3Dhttp',
		'module%3Dhttp', 'op%3Dhttp', 'starhack', '../../','..//', 'directory=http', 'dir=http', 'busca',
		'uol.com', '=http://', '=https://','=ftp://','=file://','_SESSION','CFG_ROOT','/proc/'
	);

	// Block these words found in POST requests
	$GLOBALS['ctracker_post_blacklist'] = array(
		// These two lines are for detecting hidden link spam in wikis, forums, guestbooks, etc.
		'div style=', 'overflow:auto', 'height:1px', 'width:1px', 'display:hidden',
		'overflow: auto', 'height: 1px', 'display: hidden',
		// "Common" login name from VHCS exploiters ;-)
		'starhack', 'DeLiMehmet', 'hisset', 'Hisset', 'delimert', 'MecTruy',
		'busca'
	);

	// Load email header
	$GLOBALS['ctracker_header'] = crackerTrackerLoadEmailTemplate('header');
}

// Checks for worms
function isCrackerTrackerWormDetected () {
	// Check against the whole list
	$GLOBALS['ctracker_checkworm'] = str_replace($GLOBALS['ctracker_get_blacklist'], '*', crackerTrackerQueryString());

	// If it differs to original and the *whole* request string is not in whitelist
	// then blog the attempt
	return ($GLOBALS['ctracker_checkworm'] != crackerTrackerQueryString() && (!in_array(crackerTrackerQueryString(), $GLOBALS['ctracker_whitelist'])));
}

// Checks POST data
function isCrackerTrackerPostAttackDetected () {
	// Implode recursive the whole $_POST array
	$GLOBALS['ctracker_post_track'] = implode_r('', $_POST);

	// Check for suspicious POST data
	$GLOBALS['ctracker_check_post'] = str_replace($GLOBALS['ctracker_post_blacklist'], '*', $GLOBALS['ctracker_post_track']);

	// Is it detected?
	return ((isCrackerTrackerWormDetected()) || ($GLOBALS['ctracker_check_post'] != $GLOBALS['ctracker_post_track']));
}

// Prepares a mail and send it out
function sendCrackerTrackerMail () {
	// Log the attack
	crackerTrackerLogAttack();

	// Mail content
	$mail = "Attack detected:
-----------------------------------------------------
Remote-IP       : ".determineCrackerTrackerRealRemoteAddress()."
User-Agent      : ".crackerTrackerUserAgent()."
Request-string  : ".crackerTrackerQueryString()."
Filtered string : ".$GLOBALS['ctracker_checkworm']."
Server          : ".crackerTrackerServerName()."
Script          : ".crackerTrackerScriptName()."
Referrer        : ".crackerTrackerReferer()."
-----------------------------------------------------
";

	// Send it out
	crackerTrackerSendMail($mail);

	// And die here
	crackerTrackerDie();
}

// Sends the ticket emails out
function sendCrackerTrackerTicketMails () {
	// Load user template
	$mail = crackerTrackerLoadEmailTemplate('user_add_ticket', $GLOBALS['ctracker_last_ticket']);

	// Send email to the user
	crackerTrackerSendMail($mail, $GLOBALS['ctracker_last_ticket']['ctracker_ticket_email'], getCrackerTrackerLocalized('user_add_ticket_subject'));

	// Load webmaster template
	$mail = crackerTrackerLoadEmailTemplate('webmaster_add_ticket', $GLOBALS['ctracker_last_ticket'], getCrackerTrackerConfig('ctracker_language'));

	// Send email to the user
	crackerTrackerSendMail($mail, null, getCrackerTrackerLocalized('webmaster_add_ticket_subject'));
}

// Sends a mail out
function crackerTrackerSendMail ($mail, $recipient = null, $subject = null) {
	// Construct dummy array
	$rowData = array(
		'remote_addr' => determineCrackerTrackerRealRemoteAddress(),
		'check_worm'  => $GLOBALS['ctracker_checkworm'],
		'server_name' => crackerTrackerServerName()
	);

	// Only send email if not yet found
	if (!isCrackerTrackerEntryFound($rowData)) {
		// Send the email out only in non-debug mode
		if (isCrackerTrackerDebug()) {
			// Output message
			print 'Recipient=' . $recipient . '<br />Subject=' . $subject . '<br />Text=<pre>' . $mail . '</pre>';

			// All fine
			return true;
		} elseif (!is_null($recipient)) {
			// Recipient specified
			return mail($recipient, $subject, $mail, $GLOBALS['ctracker_header']);
		} elseif (isset($GLOBALS['ctracker_email'])) {
			// Send it
			return mail($GLOBALS['ctracker_email'], 'CTracker: Attack detected!', $mail, $GLOBALS['ctracker_header']);
		} else {
			// Send it the deprecated way with constant
			return mail(constant('__CTRACKER_EMAIL'), 'CTracker: Attack detected!', $mail, $GLOBALS['ctracker_header']);
		}
	} elseif (isCrackerTrackerDebug()) {
		// Output message
		print 'Recipient=' . $recipient . '<br />Subject=' . $subject . '<br />Text=<pre>' . $mail . '</pre>';

		// All fine
		return true;
	}
}

// Sends a detected POST attack mail
function sendCrackerTrackerPostMail () {
	// Log the attack
	crackerTrackerLogAttack();

	// Mail text
	$mail = "POST-Attack detected:
-----------------------------------------------------
Remote-IP            : ".determineCrackerTrackerRealRemoteAddress()."
User-Agent           : ".crackerTrackerUserAgent()."
Request-string       : ".crackerTrackerQueryString()."
Filtered string      : ".$GLOBALS['ctracker_checkworm']."
Server               : ".crackerTrackerServerName()."
Script               : ".crackerTrackerScriptName()."
Referrer             : ".crackerTrackerReferer()."
-----------------------------------------------------
POST string          : ".$GLOBALS['ctracker_post_track']."
Filtered POST string : ".$GLOBALS['ctracker_check_post']."
-----------------------------------------------------
";

	// Send it out
	crackerTrackerSendMail($mail);

	// And die here
	crackerTrackerDie();
}

// Sleeps for a random time and aborts the script
function crackerTrackerDie () {
	// Close database link
	crackerTrackerCloseDatabaseLink();

	// Do only sleep if debug/developer mode is not enabled
	if (!isCrackerTrackerDebug()) {
		// Sleep a little to waste the attacker's time
		sleep(mt_rand(10,30));
	} // END - if

	// Bye, bye...
	if (isCrackerTrackerDebug()) {
		// With debug backtrace in debug/developer mode
		print '<pre>';
		debug_print_backtrace();
		die('</pre>');
	} else {
		// Simple die() call
		die();
	}
}

// Logs the attack attempt
function crackerTrackerLogAttack () {
	// Aquire database link
	aquireCrackerTrackerDatabaseLink();

	// By default no proxy is used
	$proxyUsed = 'N';

	// Did the attacker use a proxy?
	if (isCrackerTrackerProxyUsed()) {
		// Set it
		$proxyUsed = 'Y';
	} // END - if

	// Prepare array for database insert
	$rowData = array(
		'remote_addr'   => determineCrackerTrackerRealRemoteAddress(),
		'user_agent'    => crackerTrackerUserAgent(),
		'get_data'      => crackerTrackerQueryString(),
		'post_data'     => $GLOBALS['ctracker_post_track'],
		'check_worm'    => $GLOBALS['ctracker_checkworm'],
		'check_post'    => $GLOBALS['ctracker_check_post'],
		'server_name'   => crackerTrackerServerName(),
		'script_name'   => crackerTrackerScriptName(),
		'referer'       => crackerTrackerReferer(),
		'proxy_used'    => $proxyUsed,
		'first_attempt' => 'NOW()'
	);

	// Insert the array in database
	crackerTrackerInsertArray('ctracker_data', $rowData);
}

// Alerts the current user about malicious/suspicious traffic
function crackerTrackerAlertCurrentUser () {
	// Is there some data?
	if (isset($GLOBALS['ctracker_last_suspicious_entry'])) {
		// Does the user have a ticket?
		if (ifCrackerTrackerIpHasTicket()) {
			// Should we continue?
			if (isset($_POST['ctracker_continue'])) {
				// Set cookie
				sendCrackerTrackerCookie();

				// And redirect to same URL
				crackerTrackerRedirectSameUrl();
			} elseif (ifCrackerTrackerCookieIsSet()) {
				// Return here to normal program
				return;
			} else {
				// Load "Thank you" template
				crackerTrackerLoadTemplate('add_ticket_thanks');
			}
		} elseif ((isset($_POST['ctracker_add_ticket'])) && (!empty($_POST['name'])) && (!empty($_POST['email']))) {
			// Add the ticket
			addCrackerTrackerTicket($_POST);

			// Send the email out
			sendCrackerTrackerTicketMails();
		} else {
			// Display the form for new ticket
			crackerTrackerLoadTemplate('add_ticket');
		}
	} // END - if

	// And stop here
	die();
}

// [EOF]
?>
